(*  Title:      Tools/isac/BridgeJEdit/preliminary.sml
    Author:     Walther Neuper, JKU Linz

Preliminary code for start transition Libisabelle -- PIDE.
Will be distributed to several structures, most of which do not yet exist.
*)

signature PRELIMINARY =
sig
  val check_input: Model_Pattern.model_input_pos -> I_Model.T
  val init_ctree: theory -> string -> CTbasic_POS.ctree
  val update_state: theory -> string * (Model_Pattern.model_input_pos * References.input) ->
    CTbasic_POS.ctree -> CTbasic_POS.ctree

end

(**)
structure Preliminary(**): PRELIMINARY(**) =
struct
(**)
(** check_input **)
(*
  Implementation becomes reasonable, as soo as
 * DONE clarified, how "empty" items in Model are represented
 * parse_references_input works
 * switching \<Odot>\<Otimes> i_model for probl and meth works.
*)
fun check_input (*ctxt o-model ??*) (_: Model_Pattern.model_input_pos) = []: I_Model.T (*TODO*)
(** first trial: init or update state **)

(* at the first encounter of an Example create the following data and transfer them to ctree:
    fmz: eventually used by sub-problems, too
    origin = (O_Model, _, _): for check of missing items
    probl, meth = I_Model: for efficient check of user input
*)

(** init state **)

fun init_ctree thy example_id =
    let
      val ctxt = Proof_Context.init_global thy
      val example_id' = References_Def.explode_id example_id
      val (_, (fmz as (model, refs as (_(*thy_id*), probl_id, _)))) =
        Store.get (Know_Store.get_expls @{theory}) example_id' example_id' 
      val {model = model_patt, ...} = Problem.from_store ctxt probl_id
      val (o_model, ctxt) = O_Model.init thy model model_patt
      val i_model_empty = I_Model.init_POS ctxt o_model model_patt
    in
      CTbasic_POS.Nd (CTbasic_POS.PblObj {
        fmz = fmz, origin = (o_model, refs, TermC.empty (*TODO handle CAS_Cmd*)),
        spec = References.empty, probl = i_model_empty, meth = I_Model.empty_POS,
        ctxt = ctxt, loc = (NONE, NONE), 
        branch = CTbasic_POS.NoBranch, ostate = CTbasic_POS.Incomplete, result = (TermC.empty, [])}, [])
    end


(** update state **)

fun update_state thy (example_id, (model, refs)) CTbasic_POS.EmptyPtree = 
    (init_ctree thy example_id) |> update_state thy (example_id, (model, refs))
 | update_state _ (_, (model, refs_input))
    (CTbasic_POS.Nd (CTbasic_POS.PblObj {fmz, origin = origin as (o_model, refs_orig, _),
      spec, probl, meth, ctxt, loc, branch, ostate, result}, children)) =
      let
        val (thy_id, pbl_id, met_id) = References.select_from_input refs_input refs_orig spec
        val _ = "thy? re-parse all i_model ELSE do nothing"
        val _ = "pbl? switch from meth? check_input of probll ELSE do nothing"
        val _ = "met? switch from probl check_input of methl ELSE do nothing"
        val {model, ...} = Problem.from_store ctxt pbl_id;
(** )
        val complete_model = I_Model.complete o_model probl meth model
( **)
      in
        CTbasic_POS.Nd (CTbasic_POS.PblObj {
          fmz = fmz, origin = origin, spec = (thy_id, pbl_id, met_id), 
          probl = probl, meth = meth, (*TODO P_Model.switch_pbl_met*)
          ctxt = ctxt, loc = loc, 
          branch = branch, ostate = ostate, result = result}, children)
      end
  | update_state _  _ _ = raise ERROR "update_state: uncovered case in fun.def"

(**)end(*struct*)
