(* Title:  All tests on isac (some outcommented since Isabelle2002-->2009-2)
   Author: Walther Neuper 101001
   (c) copyright due to license terms.

   Isac's tests are organised parallel to sources: 
     $ISABELLE_ISAC_POS has same directory structure as $ISABELLE_ISAC
   plus
     $ISABELLE_ISAC_POS/Tools/isac/ADDTESTS
     $ISABELLE_ISAC_POS/Tools/isac/Minisubpbl: the Lucas-Interpreter's core functionality

Note, that only the first error in a file is shown here.
*)

section \<open>Notes on running tests\<close>
subsection \<open>Switch between running tests and updating code\<close>
text \<open>
  Isac encapsulates code as much as possible in structures without open.
  This policy conflicts with those tests, which go into functions to details
  not declared in the signatures.
\<close>
subsection \<open>Decide between running Test_Isac_Short.thy and Test_Isac.thy\<close>
text \<open>
  Some tests raise exception Size raised (line 171 of "./basis/LibrarySupport.sml")
  if run in x86_64_32 mode of Poly/ML 5.8 (which is set as default).
  This exception can be avoided by ML_system_64 = "true" in ~/.isabelle/isabisac/etc/preferences.
  A model is in the repository at ~~/etc/preferences.
  These preferences have drawbacks, however:
  * they claim more memory such that Isabelle instances canNOT run in parallel.
  * they do NOT reach Build_Isac.thy hanging in Build_Thydata.thy, see there.

  So default for Build_Isac.thy and for general testing is Test_Isac_Short.thy is x86_64_32 mode.
  From time to time full testing in Test_Isac.thy is recommended. For that purpose
  * set ML_system_64 = "true" in ~/.isabelle/isabisac/etc/preferences.

\\******************* don't forget to re-set defaults BEFORE updating code *******************//

    Note that Isabelle/jEdit re-generates the preferences file on shutdown, thus always use
    ***************** $ gedit ~/.isabelle/isabisac/etc/preferences &
\<close>

section \<open>Run the tests\<close>
text \<open>
* say "OK" to the popup asking for theories to be loaded
* watch the <Theories> window for errors in the "imports" below
\<close>

theory Test_Isac_Short
  imports Isac.Build_Isac (* note that imports are WITHOUT open struct ..*)
  (* in case of ERROR Bad theory import "Draft.Thy_All"..., open each theory one by one
     and find out, which ML_file or *.thy causes an error (might be ONLY one).
     Also backup files (#* ) recognised by jEdit cause this trouble                    *)
(*/---------------------- do Minisubpbl before ADDTESTS/All_Ctxt ------------------------------\*)
(*  "$ISABELLE_ISAC_POS/ADDTESTS/accumulate-val/Thy_All"(*but ok in editor*)*)
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/accumulate-val/Lucas_Interpreter"
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/Ctxt"
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/test-depend/Build_Test"
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/All_Ctxt"
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/Test_Units"
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/course/phst11/T1_Basics"
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/course/phst11/T2_Rewriting"
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/course/phst11/T3_MathEngine"
(**)"$ISABELLE_ISAC_POS/Tools/isac/ADDTESTS/file-depend/BuildC_Test"
(*"ADDTESTS/course/SignalProcess/Build_Inverse_Z_Transform"
   ADDTESTS/------------------------------------------- see end of tests *)
(*/~~~ these work directly from Pure, but create problems here ..
  "$ISABELLE_ISAC_POS/Pure/Isar/Keyword_ISAC.thy"           (* Malformed theory import, "keywords" ?!? *)
  "$ISABELLE_ISAC_POS/Pure/Isar/Test_Parse_Isac.thy"        (* Malformed theory import, "keywords" ?!? *)
  "$ISABELLE_ISAC_POS/Pure/Isar/Test_Parsers_Cookbook.thy"  (* Malformed theory import             ?!? *)
  "$ISABELLE_ISAC_POS/Pure/Isar/Theory_Commands"            (* Duplicate outer syntax command "ISAC"   *)
  "$ISABELLE_ISAC_POS/Pure/Isar/Downto_Synchronized"        (* re-defines / breaks structures      !!! *)
  \~~~ .. these work independently, but create problems here *)
(**)"$ISABELLE_ISAC_POS/Pure/Isar/Test_Parsers"
(**)"$ISABELLE_ISAC_POS/HOL/Tools/Sledgehammer/Try_Sledgehammer"
(*/---------------------- do Minisubpbl before ADDTESTS/All_Ctxt ------------------------------\*)
  "$ISABELLE_ISAC_POS/Tools/isac/Specify/refine"        (* setup for refine.sml   *)
  "$ISABELLE_ISAC_POS/Tools/isac/ProgLang/calculate"    (* setup for evaluate.sml *)
  "$ISABELLE_ISAC_POS/Tools/isac/Knowledge/integrate"   (* setup for integrate.sml*)
(*Test_Isac_Short.thy* )
(*\---------------------- do Minisubpbl before ADDTESTS/All_Ctxt ------------------------------/*)
(*"$ISABELLE_ISAC/Knowledge/GCD_Poly_OLD" (*not imported by Isac.thy*)        Test_Isac_Short*)
(*"$ISABELLE_ISAC/Knowledge/GCD_Poly_FP"  (*not imported by Isac.thy*)        Test_Isac_Short*)
(*\---------------------- do Minisubpbl before ADDTESTS/All_Ctxt ------------------------------/*)
( *Test_Isac_Short.thy*)
  "$ISABELLE_ISAC_POS/Tools/isac/BridgeJEdit/Test_VSCode_Example"

begin

declare [[ML_print_depth = 20]] (*otherwise buffer overflow*)

ML \<open>open ML_System\<close>
ML \<open>
(** )open Refine;( * overwrites import "...Specify/refine" above, see "open Refine" in refine.sml *)
  open Store;
  open M_Match;
  open P_Model;
  open Kernel;
  open Math_Engine;
  open Test_Code;              Test_Code.init_calc @{context};
  open LItool;                 arguments_from_model;
  open Sub_Problem;
  open Fetch_Tacs;
  open Step
  open Env;
  open LI;                     scan_dn;
  open Istate;
  open Error_Pattern;
  open Error_Pattern_Def;
  open Specification;
  open Ctree;                  append_problem;
  open Pos;
  open Program;
  open Prog_Tac;
  open Tactical;
  open Prog_Expr;
  open Auto_Prog;              rule2stac;
  open Input_Descript;
  open Specify;
  open Specify;
  open Step_Specify;
  open Step_Solve;
  open Step;
  open Solve;                  (* NONE *)
  open ContextC;               transfer_asms_from_to;
  open Tactic;                 (* NONE *)
  open I_Model;
  open O_Model;
  open P_Model;                (* NONE *)
  open Rewrite;
  open Eval;                   get_pair;
  open TermC;
  open Rule;
  open Rule_Set;               Sequence;
  open Eval_Def
  open ThyC
  open ThmC_Def
  open ThmC
  open Rewrite_Ord
  open UnparseC;

  Know_Store.set_ref_last_thy @{theory};
  (*otherwise ERRORs in pbl-met-hierarchy.sml, refine.sml, evaluate.sml*)
\<close>

section \<open>code for copy & paste ===============================================================\<close>
text \<open>
  declare [[show_types]] 
  declare [[show_sorts]]
  find_theorems "?a <= ?a"
  
  print_theorems
  print_facts
  print_statement ""
  print_theory
  ML_command \<open>Pretty.writeln prt\<close>
  declare [[ML_print_depth = 999]]
  declare [[ML_exception_trace]]
\<close>
ML \<open>
\<close> ML \<open>
"~~~~~ fun xxx , args:"; val () = ();
"~~~~~ and xxx , args:"; val () = ();
"~~~~~ fun xxx recursive, args:"; val () = ();
(*if*) (*then*); (*else*); (*andalso*)  (*case*)  (*in*) (*end*);
"xx"
^ "xxx"   (*+*) (*+++*)  (*isa*) (*isa2*) (**);
val return_XXXXX = "XXXXX"
\<close> ML \<open> (*//----------- step into XXXXX -----------------------------------------------------\\*)
(*//------------------ step into XXXXX -----------------------------------------------------\\*)
\<close> ML \<open> (*||----------- contine-- XXXXX -------------------------------------------------------*)
(*||------------------ contine-- XXXXX -------------------------------------------------------*)
(*\\------------------ step into XXXXX -----------------------------------------------------//*)
\<close> ML \<open> (*\\----------- step into XXXXX -----------------------------------------------------//*)
val "XXXXX" = return_XXXXX;

(* final test ... ----------------------------------------------------------------------------*)

\<close> ML \<open> (*//----------- build fun XXXXX -----------------------------------------------------\\*)
(*//------------------ build fun XXXXX -----------------------------------------------------\\*)
(*\\------------------ build fun XXXXX -----------------------------------------------------//*)
\<close> ML \<open> (*\\----------- build fun XXXXX -----------------------------------------------------//*)

\<close> ML \<open> (*//----------- inserted hidden code ------------------------------------------------\\*)
(*//------------------ inserted hidden code ------------------------------------------------\\*)
(*\\------------------ inserted hidden code ------------------------------------------------//*)
\<close> ML \<open> (*\\----------- inserted hidden code ------------------------------------------------//*)

\<close> ML \<open> (*//----------- setup test data for XXXXX -------------------------------------------\\*)
(*//------------------ setup test data for XXXXX -------------------------------------------\\*)
(*\\------------------ setup test data for XXXXX -------------------------------------------//*)
\<close> ML \<open> (*\\----------- setup test data for XXXXX -------------------------------------------//*)
\<close>
ML \<open>
\<close> ML \<open>

\<close> ML \<open>
\<close>

section \<open>trials with Isabelle's functions\<close>
  ML \<open>"%%%%%%%%%%%%%%%%% start Isabelle %%%%%%%%%%%%%%%%%%%%%%%";\<close>
  ML_file "$ISABELLE_ISAC_POS/Pure/General/alist.ML"
  ML_file "$ISABELLE_ISAC_POS/Pure/General/basics.ML"
  ML_file "$ISABELLE_ISAC_POS/Pure/General/scan.ML"
  ML_file "$ISABELLE_ISAC_POS/Pure/PIDE/xml.ML"
  ML \<open>"%%%%%%%%%%%%%%%%% end Isabelle %%%%%%%%%%%%%%%%%%%%%%%%%";\<close>

section \<open>test ML Code of isac\<close>
subsection \<open>basic code first\<close>
  ML \<open>"%%%%%%%%%%%%%%%%% start ProgLang.thy %%%%%%%%%%%%%%%%%%%";\<close>
  ML_file "BaseDefinitions/base-definitions.sml"
  ML_file "BaseDefinitions/libraryC.sml"
  ML_file "BaseDefinitions/rule-def.sml"
  ML_file "BaseDefinitions/eval-def.sml"
  ML_file "BaseDefinitions/rewrite-order.sml"
  ML_file "BaseDefinitions/theoryC.sml"
  ML_file "BaseDefinitions/rule.sml"
  ML_file "BaseDefinitions/thmC-def.sml"
  ML_file "BaseDefinitions/model-pattern.sml"
  ML_file "BaseDefinitions/error-fill-def.sml"
  ML_file "BaseDefinitions/rule-set.sml"
  ML_file "BaseDefinitions/check-unique.sml"
(*called by Know_Store..*)
  ML_file "BaseDefinitions/calcelems.sml"
  ML_file "BaseDefinitions/termC.sml"
  ML_file "BaseDefinitions/substitution.sml"
  ML_file "BaseDefinitions/contextC.sml"(*sometimes needs separation into ML blocks for evaluation*)
  ML_file "BaseDefinitions/environment.sml"
(**)ML_file "BaseDefinitions/kestore.sml"(*setup in ADDTEST/accumulate-val/lucas_interpreter.sml*)
(*---------------------- do Minisubpbl before ADDTESTS/All_Ctxt --------------------------------
  ---------------------- do Minisubpbl before ADDTESTS/All_Ctxt --------------------------------*)

  ML_file "ProgLang/calculate.sml"
  ML_file "ProgLang/evaluate.sml"       (* requires setup from calculate.thy                    *)
  ML_file "ProgLang/listC.sml"
  ML_file "ProgLang/prog_expr.sml"
  ML_file "ProgLang/program.sml"
  ML_file "ProgLang/prog_tac.sml"
  ML_file "ProgLang/tactical.sml"
  ML_file "ProgLang/auto_prog.sml"
(*---------------------- do Minisubpbl before ADDTESTS/All_Ctxt --------------------------------
  ---------------------- do Minisubpbl before ADDTESTS/All_Ctxt --------------------------------*)

subsection \<open>basic functionality on simple example first\<close>
  ML_file "Minisubpbl/000-comments.sml"
  ML_file "Minisubpbl/100-init-rootpbl.sml"
  ML_file "Minisubpbl/150a-add-given-Maximum.sml"
  ML_file "Minisubpbl/150-add-given-Equation.sml"
  ML_file "Minisubpbl/200-start-method-NEXT_STEP.sml"
  ML_file "Minisubpbl/200-start-method.sml"
  ML_file "Minisubpbl/250-Rewrite_Set-from-method.sml"
  ML_file "Minisubpbl/300-init-subpbl-NEXT_STEP.sml"
  ML_file "Minisubpbl/300-init-subpbl.sml"
  ML_file "Minisubpbl/400-start-meth-subpbl.sml"
  ML_file "Minisubpbl/450-Rewrite_Set_Inst.sml"
  ML_file "Minisubpbl/470-Check_elementwise-NEXT_STEP.sml"
  ML_file "Minisubpbl/490-nxt-Check_Postcond.sml"
  ML_file "Minisubpbl/500-met-sub-to-root.sml"
  ML_file "Minisubpbl/530-error-Check_Elementwise.sml"
  ML_file "Minisubpbl/600-postcond-NEXT_STEP.sml"
  ML_file "Minisubpbl/600-postcond.sml"
  ML_file "Minisubpbl/700-interSteps.sml"
  ML_file "Minisubpbl/710-interSteps-short.sml"
  ML_file "Minisubpbl/790-complete-NEXT_STEP.sml"
  ML_file "Minisubpbl/790-complete.sml"
  ML_file "Minisubpbl/800-append-on-Frm.sml"

subsection \<open>further functionality alongside batch build sequence\<close>
  ML_file "MathEngBasic/thmC.sml"
  ML_file "MathEngBasic/problem.sml"
  ML_file "MathEngBasic/rewrite.sml"
  ML_file "MathEngBasic/tactic.sml"
  ML_file "MathEngBasic/ctree.sml" (*if red, get the file into a text buffer -- this might clear*)
  ML_file "MathEngBasic/references.sml"
  ML_file "MathEngBasic/calculation.sml"

  ML_file "Specify/formalise.sml"
  ML_file "Specify/o-model.sml"
  ML_file "Specify/i-model.sml"       (* (BROKEN!) test on elementwise input to lists*)
  ML_file "Specify/pre-conditions.sml"
  ML_file "Specify/p-model.sml"
  ML_file "Specify/m-match.sml"
  ML_file "Specify/refine.sml"        (* requires setup from refine.thy *)
  ML_file "Specify/test-out.sml"
  ML_file "Specify/specify-step.sml"
  ML_file "Specify/specification.sml"
  ML_file "Specify/cas-command.sml"
  ML_file "Specify/p-spec.sml"
(*ML_file "Specify/specify.sml ---with M_Model.by_i_models broken elementwise input to lists---*)
  ML_file "Specify/sub-problem.sml"
  ML_file "Specify/step-specify.sml"

  ML_file "Interpret/istate.sml"
  ML_file "Interpret/error-pattern.sml"
  ML_file "Interpret/li-tool.sml"(*---with M_Model.by_i_models broken elementwise input to lists---*)
  ML_file "Interpret/lucas-interpreter.sml"
  ML_file "Interpret/step-solve.sml"

  ML_file "MathEngine/me-misc.sml"
  ML_file "MathEngine/fetch-tactics.sml"
  ML_file "MathEngine/solve.sml"
  ML_file "MathEngine/step.sml"
  ML_file "MathEngine/mathengine-stateless.sml"
  ML_file "MathEngine/messages.sml"
  ML_file "MathEngine/states.sml"

  ML_file "BridgeLibisabelle/mathml.sml"           (*part.*)
  ML_file "BridgeLibisabelle/pbl-met-hierarchy.sml"
  ML_file "BridgeLibisabelle/interface-xml.sml"     (*TODO after 2009-2*)
  ML_file "BridgeLibisabelle/interface.sml"

(**) (* evaluated in Build_Isac.thy already *)
  ML_file "BridgeJEdit/e-collect.sml"
  ML_file "BridgeJEdit/user-model.sml"
  ML_file "BridgeJEdit/template.sml"
  ML_file "BridgeJEdit/preliminary.sml"
  ML_file "BridgeJEdit/calculation.sml"
  ML_file "BridgeJEdit/vscode-example.sml"
(**)

  ML_file "Knowledge/delete.sml"
  ML_file "Knowledge/descript.sml"
  ML_file "Knowledge/simplify.sml"
  ML_file "Knowledge/poly-1.sml"
(*ML_file "Knowledge/poly-2.sml"                                                Test_Isac_Short*)
  ML_file "Knowledge/gcd_poly_ml.sml"
  ML_file "Knowledge/rational-1.sml"
(*ML_file "Knowledge/rational-2.sml"                                            Test_Isac_Short*)
  ML_file "Knowledge/equation.sml"
  ML_file "Knowledge/root.sml"
  ML_file "Knowledge/lineq.sml"
(*ML_file "Knowledge/rooteq.sml"    some complicated equations not recovered from 2002 *)
(*ML_file "Knowledge/rateq.sml"     some complicated equations not recovered----Test_Isac_Short*)
  ML_file "Knowledge/rootrat.sml"
  ML_file "Knowledge/rootrateq.sml"(*one complicated equations not recovered from 2002 *)
(*ML_file "Knowledge/partial_fractions.sml"  hangs with ML_system_64 = "true"---Test_Isac_Short*)
  ML_file "Knowledge/polyeq-1.sml"
(*ML_file "Knowledge/polyeq-2.sml"                                              Test_Isac_Short*)
(*ML_file "Knowledge/rlang.sml"     much to clean up, similar tests in other files     *)
  ML_file "Knowledge/calculus.sml"
  ML_file "Knowledge/trig.sml"
(*ML_file "Knowledge/logexp.sml"    not included as stuff for presentation of authoring*) 
  ML_file "Knowledge/diff.sml"
  ML_file "Knowledge/integrate.sml"
  ML_file "Knowledge/eqsystem-1.sml"
  ML_file "Knowledge/eqsystem-1a.sml"
(*ML_file "Knowledge/eqsystem-2.sml"                                          Test_Isac_Short*)
  ML_file "Knowledge/test.sml"
  ML_file "Knowledge/polyminus.sml"
  ML_file "Knowledge/vect.sml"
  ML_file "Knowledge/diff-app.sml"        (* postponed to dev. specification | TP-prog. *)
  ML_file "Knowledge/biegelinie-1.sml"
(*ML_file "Knowledge/biegelinie-2.sml"                                          Test_Isac_Short*)
(*ML_file "Knowledge/biegelinie-3.sml"                                          Test_Isac_Short*)
(*ML_file "Knowledge/biegelinie-4.sml" (*with M_Model.by_i_models broken in |> me'*)*)
  ML_file "Knowledge/algein.sml"
  ML_file "Knowledge/diophanteq.sml"
(*ML_file "Knowledge/inverse_z_transform.sml"hangs with ML_system_64 = "true"---Test_Isac_Short*)
  ML_file "Knowledge/inssort.sml"
  ML_file "Knowledge/isac.sml"

  ML_file "Test_Code/test-code.sml"

section \<open>further tests additional to src/.. files\<close>
  ML_file "BridgeLibisabelle/use-cases.sml"

  ML \<open>"%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%";\<close>
  ML \<open>"%%%%%%%%%%%%%%%%% end Test_Isac %%%%%%%%%%%%%%%%%%%%%%%%";\<close>
  ML \<open>"%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%";\<close>
ML \<open>
\<close> ML \<open>

\<close> ML \<open>
\<close>

section \<open>history of tests\<close>
text \<open>
  Systematic regression tests have been introduced to isac development in 2003.
  Sanity of the regression tests suffers from updates following Isabelle development,
  which mostly exceeded the resources available in isac's development.

  The survey below shall support to efficiently use the tests for isac 
  on different Isabelle versions. Conclusion in most cases will be: 

  !!! Use most recent tests or go back to the old notebook
      with isac on Isabelle2002. !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
\<close>


subsection \<open>isac on Isabelle2017\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  * Add further signatures, separate structures and cleanup respective files.
  * Show feasibility of moving Isac programs to partial_function, i.e. to the function package.
  * Clean theory dependencies.
  * Start preparing shift from isac-java to Isabelle/jEdit.
\<close>
subsubsection \<open>State of tests: unchanged\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  last changeset with Test_Isac 925fef0f4c81
  first changeset with Test_Isac bbb414976dfe
\<close>

subsection \<open>isac on Isabelle2015\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  * Add signatures from top of thy-hierarchy down to Interpret (not ProgLang).
    This complicates Test_Isac, see "Prepare running tests" above.
  * Remove TTY interface.
  * Re-activate insertion sort.
\<close>
subsubsection \<open>State of tests: unchanged\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  last changeset with Test_Isac 2f1b2854927a
  first changeset with Test_Isac ???
\<close>

subsection \<open>isac on Isabelle2014\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  migration from "isabelle tty" --> libisabelle
\<close>

subsection \<open>isac on Isabelle2013-2\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  reactivated context_thy
\<close>
subsubsection \<open>State of tests\<close>
text \<open>
  TODO
\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  TODO
  :
  : isac on Isablle2013-2
  :
  Changeset: 55318 (03826ceb24da) merged
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-12-12 14:27:37 +0100 (7 minutes)
\<close>

subsection \<open>isac on Isabelle2013-1\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  Isabelle2013-1 was replaced within a few weeks due to problems with the document model;
  no significant development steps for ISAC.
\<close>
subsubsection \<open>State of tests\<close>
text \<open>
  See points in subsection "isac on Isabelle2011", "State of tests".
\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  Changeset: 55283 (d6e9a34e7142) notes for resuming work on Polynomial.thy
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-12-03 18:13:31 +0100 (8 days)
  :
  : isac on Isablle2013-1
  :
  Changeset: 55279 (130688f277ba) Isabelle2013 --> 2013-1: Test_Isac perfect
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-11-21 18:12:17 +0100 (2 weeks)

\<close>

subsection \<open>isac on Isabelle2013\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  # Oct.13: replaced "axioms" by "axiomatization"
  # Oct.13: Mathias Lehnfeld started removing Unsynchornized.ref
  # Sep.13: integrated gcd_poly (functional, without Unsychronized.ref) into
    simplification of multivariate rationals (without improving the rulesets involved).
\<close>
subsubsection \<open>Run tests\<close>
text \<open>
  Is standard now; this subsection will be discontinued under Isabelle2013-1
\<close>
subsubsection \<open>State of tests\<close>
text \<open>
  See points in subsection "isac on Isabelle2011", "State of tests".
  # re-activated listC.sml
\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  changeset 52174:8b055b17bd84 --- removed all code concerned with "castab = Unsynchronized.ref"
  User: Mathias Lehnfeld <s1210629013@students.fh-hagenberg.at>
  Date: Tue Nov 19 22:23:30 2013 +0000
  :
  : isac on Isablle2013 
  :
  Changeset: 52061 (4ecea2fcdc2c) --- Build_Isac.thy runs on Isabelle2013
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-07-15 08:28:50 +0200 (4 weeks)
\<close>

subsection \<open>isac on Isabelle2012\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  isac on Isabelle2012 is considered just a transitional stage
  within the update from Isabelle2011 to Isabelle2013; thus no further development of isac;
  For considerations on the transition see 
  $ISABELLE_ISAC/Knowledge/Build_Thydata/thy, section "updating isac..".
\<close>
subsubsection \<open>Run tests\<close>
text \<open>
$ cd /usr/local/isabisac12/
$ ./bin/isabelle jedit -l HOL src/Tools/isac/Build_Isac.thy
$ ./bin/isabelle jedit -l Isac test/Tools/isac/Test_Isac.thy
\<close>
subsubsection \<open>State of tests\<close>
text \<open>
  At least the tests from isac on Isabelle2011 run again.
  However, Test_Isac.thy shows erratic behaviour; no errors are obtained when scrolling 
  in parallel with evaluation.

  Counting "error in kernel" for Frontend/interface.sml (the tests considered most significant)
  yields 69 hits, some of which were already present before Isabelle2002-->2009-2
  (i.e. on the old notebook from 2002).

  Now many tests with (*...=== inhibit exn ...*) give a reason or at least the origin:
  # === inhibit exn WN1130621 Isabelle2012-->13 !thehier! === ...see Build_Thydata.thy
  # === inhibit exn AK110726 === ...reliable work by Alexander Kargl, most likely go back to 2002
  # === inhibit exn WN1130701 broken at Isabelle2002 --> 2009-2 === , most likely go back to 2002
  Reasons for outcommented tests are also found in Test_Isac.thy near the respective file.sml.

  Some tests have been re-activated (e.g. error patterns, fill patterns).
\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  Changeset: 52051 (35751d90365e) end of improving tests for isac on Isabelle2012
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2013-07-11 16:58:31 +0200 (4 weeks)
  :
  : isac on Isablle2012 
  :
  Changeset: 48757 (74eb3dfc33cc) updated src from Isabelle2011 to Isabelle2012
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2012-09-24 18:35:13 +0200 (8 months)
  ------------------------------------------------------------------------------
  Changeset: 48756 (7443906996a8) merged
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2012-09-24 18:15:49 +0200 (8 months)
\<close>

subsection \<open>isac on Isabelle2011\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  isac's mathematics engine has been extended by two developments:
  (1) Isabelle's contexts were introduced by Mathias Lehnfeld
  (2) Z_Transform was introduced by Jan Rocnik, which revealed
    further errors introduced by (1).
  (3) "error patterns" were introduced by Gabriella Daroczy
  Regressions tests have been added for all of these.
\<close>
subsubsection \<open>Run tests\<close>
text \<open>
  $ cd /usr/local/isabisac11/
  $ ./bin/isabelle jedit -l HOL src/Tools/isac/Build_Isac.thy
  $ ./bin/isabelle jedit -l Isac test/Tools/isac/Test_Isac.thy
\<close>
subsubsection \<open>State of tests\<close>
text \<open>
  Systematic efforts outcommented less significant tests by (*...=== inhibit exn ...*) 
  and sometimes give reasons for failing tests.
  (*...=== inhibit exn AK...*) was done by Alexander Kargl; this is reliable
  work, some of which couldn't be revised (and renamed) by WN and thus survived some time.

  The most signification tests (in particular Frontend/interface.sml) run,
  however, many "error in kernel" are not caught by an exception.
  ------------------------------------------------------------------------------
  After the changeset below Test_Isac worked with check_unsynchronized_ref ():
  ------------------------------------------------------------------------------
  Changeset: 42457 (ca691a84b81a) PROVISIONALLY MADE TESTS RUN with Unsynchronized.ref
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2012-08-06 10:38:11 +0200 (11 months)


  The list below records TODOs while producing an ISAC kernel for 
  gdaroczy and jrocnik, wich could NOT be done before all tests are RUNNING
  (so to be resumed with Isabelle2013-1):
  ############## WNxxxxxx.TODO can be found in sources ##############
  --------------------------------------------------------------------------------
  WN111013.TODO: lots of cleanup/removal in test/../Test.thy
  --------------------------------------------------------------------------------
  WN111013.TODO: remove concept around "fun implicit_take", lots of troubles with 
  this special case (see) --- why not nxt = Model_Problem here ? ---
  --------------------------------------------------------------------------------
  WN111014.TODO calculate_Poly < calculate_Rational < calculate_RootRat, see test/
  ... FIRST redesign 
  # simplify_* , *_simp_* 
  # norm_* 
  # calc_* , calculate_*  ... require iteration over all rls ...
  ... see --- val rls = calculate_RootRat > calculate_Rational --- CONTINUE !
  --------------------------------------------------------------------------------
  WN111014.TODO fun prep_rls | !!!use this function in ruleset' := !!!
  --------------------------------------------------------------------------------
  WN120314 changeset a393bb9f5e9f drops root equations.
  see test/Tools/isac/Knowledge/rootrateq.sml 
  --------------------------------------------------------------------------------
  WN120317.TODO changeset 977788dfed26 dropped rateq:
  # test --- repair NO asms from rls RatEq_eliminate --- shows error from 2002
  # test --- solve (1/x = 5, x) by me --- and --- x / (x ^ 2 - 6 * x + 9) - ...:    
    investigation Check_elementwise stopped due to too much effort finding out,
    why Check_elementwise worked in 2002 in spite of the error.
  --------------------------------------------------------------------------------
  WN120317.TODO postponed test/../ratinal,ratinal2.sml to joint work with dmeindl 
  --------------------------------------------------------------------------------
  WN120317.TODO found by test --- interSteps for Schalk 299a --- that 
    NO test with 'interSteps' is checked properly (with exn on changed behaviour)
  --------------------------------------------------------------------------------
  WN120317.TODO test --- Matthias Goldgruber 2003 rewrite orders --- has
    a newly outcommented test where rewrite_set_ make_polynomial --> NONE
  --------------------------------------------------------------------------------
  WN120320.TODO check-improve rlsthmsNOTisac:
  DONE make test --- old compute rlsthmsNOTisac by eq_thmI'
  DONE compare rlsthmsNOTisac in thms-survey-Isa02-Isa09-2.sml .. Isac.thy 
  FOUND 120321: Theory.axioms_of doesnt find LENGTH_CONS etc, thus are in Isab
  # mark twice thms (in isac + (later) in Isabelle) in Isac.thy
  --------------------------------------------------------------------------------
  WN120320.TODO rlsthmsNOTisac: replace twice thms ^
  --------------------------------------------------------------------------------
  WN120320.TODO rlsthmsNOTisac: reconsider design of sym_* thms, see test
  --- OLD compute rlsthmsNOTisac by eq_thmID ---: some are in isab, some in isac.
  --------------------------------------------------------------------------------
  WN120321.TODO rearrange theories:
    Knowledge
      :
      Prog_Expr.thy
      ///Input_Descript.thy --> ProgLang
      Delete.thy   <--- first_Knowledge_thy (*mv to Prog_Expr.thy*)
    ProgLang: restructure Build_Isac.thy such that no xmlsrc.thy, ProgLang.thy
          Interpret.thy are generated (simplifies xml structure for theories)
      Program.thy
      Tools.thy
      ListC.thy    <--- first_Proglang_thy
  --------------------------------------------------------------------------------
  WN120321.TODO reanimate test/../simplify.sml: CAS-command Simplify
      EXN "simplify.sml: diff.behav. CAScmd: Simplify (2*a + 3*a)"
  broken during work on thy-hierarchy
  --------------------------------------------------------------------------------
  WN120321.TODO LAST in IsacScripts + in IsacKnowledge, see
  test --- the_hier (get_thes ()) (collect_thydata ())---
  --------------------------------------------------------------------------------
  WN120405a.TODO src/../pbl-met-hierarchy.sml: fun pbl2term--> Isac' instead Isac_
  !!add mutual crossreferences to ?fun headline??? where the same has to be done:
  !!OR BETTER: use only 2 functions for adding/removing "'" to/from thy!!
  --------------------------------------------------------------------------------
  WN120411 scanning html representation of newly generated knowledge:
  * thy:
  ** Theorems: only "Proof of the theorem" (correct!)
               and "(c) isac-team (math-autor)"
  ** Rulesets: only "Identifier:///"
               and "(c) isac-team (math-autor)"
  ** IsacKnowledge: link to dependency graph (which needs to be created first)
  ** IsacScripts --> ProgramLanguage
  *** Tools: Theorems: arity_type_cp, arity_type_nam, ... wegnehmen
  
  * pbl: OK !?!
  * met: OK !?!
  * exp: 
  ** Z-Transform is missing !!!
  ** type-constraints !!!
  --------------------------------------------------------------------------------
  WN120417: merging xmldata revealed:
  ..............NEWLY generated:........................................
  <THEOREMDATA>
    <GUH> thy_isab_Fun-thm-o_apply </GUH>
    <STRINGLIST>
      <STRING> Isabelle </STRING>
      <STRING> Fun </STRING>
      <STRING> Theorems </STRING>
      <STRING> o_apply </STRING>
    </STRINGLIST>
      <MATHML>
        <ISA> (?f o ?g) ?x = ?f (?g ?x) </ISA>
      </MATHML>  <PROOF>
      <EXTREF>
        <TEXT> Proof of the theorem </TEXT>
        <URL> http://www.ist.tugraz.at/projects/isac/www/kbase/thy/browser_info/HOL/HOL-Real/Isac/Fun.html </URL>
      </EXTREF>
    </PROOF>
    <EXPLANATIONS> </EXPLANATIONS>
    <MATHAUTHORS>
      <STRING> Isabelle team, TU Munich </STRING>
    </MATHAUTHORS>
    <COURSEDESIGNS>
    </COURSEDESIGNS>
  </THEOREMDATA>
  ..............OLD FORMAT:.............................................
  <THEOREMDATA>
    <GUH> thy_isab_Fun-thm-o_apply </GUH>
    <STRINGLIST>
      <STRING> Isabelle </STRING>
      <STRING> Fun </STRING>
      <STRING> Theorems </STRING>
      <STRING> o_apply </STRING>
    </STRINGLIST>
    <THEOREM>
      <ID> o_apply </ID>
      <MATHML>
        <ISA> (?f o ?g) ?x = ?f (?g ?x) </ISA>
      </MATHML>
    </THEOREM>
    <PROOF>
      <EXTREF>
        <TEXT> Proof of the theorem </TEXT>
        <URL> http://www.ist.tugraz.at/projects/isac/www/kbase/thy/browser_info/HOL/HOL-Real/Isac/Fun.html </URL>
      </EXTREF>
    </PROOF>
    <EXPLANATIONS> </EXPLANATIONS>
    <MATHAUTHORS>
      <STRING> Isabelle team, TU Munich </STRING>
    </MATHAUTHORS>
    <COURSEDESIGNS>
    </COURSEDESIGNS>
  </THEOREMDATA>
  --------------------------------------------------------------------------------
\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  isac development was done between these changesets:
  ------------------------------------------------------------------------------
  Changeset: 42519 (1f3b4270363e) meeting dmeindl: added missing files
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2012-09-24 16:39:30 +0200 (8 months)
  :
  : isac on Isablle2011
  :
  Changeset:41897 (355be7f60389) merged isabisac with Isabelle2011
  Branch: decompose-isar 
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2011-02-25 13:04:56 +0100 (2011-02-25)
  ------------------------------------------------------------------------------
\<close>

subsection \<open>isac on Isabelle2009-2\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  In 2009 the update of isac from Isabelle2002 started with switching from CVS to hg.
  The update was painful (bridging 7 years of Isabelle development) and cut short 
  due to the start of introducing Isabelle's contexts (Mathias Lehnfeld) and
  going on to Isabelle2011 although most of the tests did not run.
\<close>
subsubsection \<open>Run tests\<close>
text \<open>
  WN131021 this is broken by installation of Isabelle2011/12/13,
  because all these write their binaries to ~/.isabelle/heaps/..

  $ cd /usr/local/isabisac09-2/
  $ ./bin/isabelle emacs -l HOL src/Tools/isac/Build_Isac.thy
  $ ./bin/isabelle emacs -l Isac src/Tools/isac/Test_Isac.thy
  NOT THE RIGHT VERSION.....    test/Tools/isac/Test_Isac.thy !!!
\<close>
subsubsection \<open>State of tests\<close>
text \<open>
  Most tests are broken by the update from Isabelle2002 to Isabelle2009-2.
\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  isac development was done between these changesets:
  ------------------------------------------------------------------------------
  Changeset: 38115 (940a5feea094) Little improvements of isac-plugin
  Branch: decompose-isar 
  User: Marco Steger <m.steger@student.tugraz.at>
  Date: 2011-02-06 18:30:28 +0100 (2011-02-06)
  :
  : isac on Isablle2009-2
  :
  Changeset: 37870 (5100a9c3abf8) created branch isac-from-Isabelle2009-2
  Branch: isac-from-Isabelle2009-2 
  User: Walther Neuper <neuper@ist.tugraz.at>
  Date: 2010-07-21 09:59:35 +0200 (2010-07-21)
  ------------------------------------------------------------------------------
\<close>

subsection \<open>isac on Isabelle2002\<close>
subsubsection \<open>Summary of development\<close>
text \<open>
  From 1999 to 2010 all the basic functionality (except "ctxt" and "error pattern")
  of isac's mathematics engine has been implemented.
\<close>
subsubsection \<open>Run tests\<close>
subsubsection \<open>State of tests\<close>
text \<open>
  All tests work on an old notebook (the right PolyML coudn't be upgraded to more
  recent Linux versions)
\<close>
subsubsection \<open>Changesets of begin and end\<close>
text \<open>
  Up to the above Mercurial changeset 5100a9c3abf8 isac used CVS;
  see https://intra.ist.tugraz.at/hg/isac containing a conversion to Mercurial.
\<close>
ML \<open>
\<close> ML \<open>

\<close> ML \<open>
\<close>
end
(*========== inhibit exn 130719 Isabelle2013 ===================================
============ inhibit exn 130719 Isabelle2013 =================================*)

(*-.-.-.-.-.-isolate response.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.
  -.-.-.-.-.-.-isolate response.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.*)

